﻿Bio_Tint = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDebug-2.0", "AceConsole-2.0", "AceDB-2.0")

local defaults = {
	limit = 0.65,
	cutoff = 0.2,
	speed = 0.5,
}

local options = {
	type = "group",
	args = {
		limit = {
			type = "range",
			name = "Limit",
			desc = "The percentage of total health where the screen tint starts",
			get = function() return Bio_Tint.db.profile.limit end,
			set = "SetLimit",
			min = 0.5,
			max = 0.9,
			step = 0.01,
			isPercent = true
		},
		cutoff = {
			type = "range",
			name = "Cutoff",
			desc = "The percentage of total health where the screen is at full tint",
			get = function() return Bio_Tint.db.profile.cutoff end,
			set = "SetCutoff",
			min = 0,
			max = 0.4,
			step = 0.01,
			isPercent = true
		},
		speed = {
			type = "range",
			name = "Speed",
			desc = "The speed at which the tint changes",
			get = function() return Bio_Tint.db.profile.speed end,
			set = function(speed) Bio_Tint.db.profile.speed = speed end,
			min = 0.01,
			max = 5,
		},
	}
}

function Bio_Tint:OnInitialize()
--	self:SetDebugging(true)

	self:RegisterDB("Bio_TintDB")
	self:RegisterDefaults("profile", defaults)

	self:RegisterChatCommand({"/bio_tint"}, options);
end

function Bio_Tint:OnEnable()
	if not self.borders then
		self.borders = self.Borders()
--		self.borders:SetTexture("Interface\\Addons\\Bio_Tint\\Textures\\Gradient-WRed75-25-128.tga", 0.26)
		self.borders:SetTexture("Interface\\Addons\\Bio_Tint\\Textures\\Gradient-WRed90-20-128.tga", 0.26)
		self.borders:SetBlendMode("MOD")

		self.updateFrame = CreateFrame("Frame", nil, UIParent)
		self.updateFrame:SetScript("OnUpdate", function() self:OnUpdate() end)
	end

	self:RegisterEvent("UNIT_HEALTH", "UpdateHealth");
	self:RegisterEvent("UNIT_MAXHEALTH", "UpdateHealth");

	self:SetTargetAmount(0, true)
	self:UpdateHealth('player')
end

function Bio_Tint:OnDisable()
	self:SetTargetAmount(0, true)
end

function Bio_Tint:OnProfileEnable()
	if self:IsActive() then
		self:UpdateHealth('player')
	end
end

function Bio_Tint:OnUpdate()
	local time = GetTime()
	local target = self.targetAmount
	local current = self.currentAmount
	local dist = target - current
	local adist = math.abs(dist)
	local maxMove = (time - self.lastUpdate) * self.db.profile.speed
	if adist <= maxMove then
		current = target
		self:Debug("stopping updates (OnUpdate)")
		self.updateFrame:Hide()
	else
		current = current + maxMove * (dist / adist)
		self.lastUpdate = time
	end
	self.currentAmount = current
	self.borders:SetAmount(current)
end

function Bio_Tint:SetTargetAmount(amount, now)
	self:Debug('SetTargetAmount', amount, now)
	if now then
		self.currentAmount = amount
		self.targetAmount = amount
		self.borders:SetAmount(amount)
		self:Debug("stopping updates (SetTargetAmount)")
		self.updateFrame:Hide()
	else
		self.targetAmount = amount
		if self.currentAmount ~= amount and not self.updateFrame:IsShown() then
			self:Debug("starting updates (SetTargetAmount)")
			self.updateFrame:Show()
			self.lastUpdate = GetTime()
		end
	end
end

function Bio_Tint:NotifyHealth(health)
	self:Debug('NotifyHealth', health)
	local high = self.db.profile.limit
	local low = self.db.profile.cutoff
	if health >= high then
		self:SetTargetAmount(0)
	elseif health <= low then
		self:SetTargetAmount(1)
	else
		local dist = high - low
		self:SetTargetAmount(1 - (health - low)/dist)
	end
end

function Bio_Tint:UpdateHealth(unit)
	if unit == "player" then
		if UnitIsDeadOrGhost(unit) then
			self:NotifyHealth(1)
		else
			self:NotifyHealth(UnitHealth(unit) / UnitHealthMax(unit))
		end
	end
end

function Bio_Tint:SetLimit(limit)
	self.db.profile.limit = limit
	self:UpdateHealth('player')
end

function Bio_Tint:SetCutoff(cutoff)
	self.db.profile.cutoff = cutoff
	self:UpdateHealth('player')
end
